//--- Connection with the main file of the Expert Advisor
#include "../../Experts/FUGT/FUGT.mq5"
//--- Include custom libraries
#include "../../Include/FUGT/Auxiliary.mqh"
#include "../../Include/FUGT/Enums.mqh"
#include "../../Include/FUGT/Errors.mqh"
#include "../../Include/FUGT/ToString.mqh"
#include "../../Include/FUGT/TradeFunctions.mqh"
#include "../../Include/FUGT/TradeSignals.mqh"
//+------------------------------------------------------------------+
//| Filling the array of symbols                                     |
//+------------------------------------------------------------------+
void GetSymbols()
  {
   Symbols[0]=GetSymbolByName(InpSymbol_01);
   //Symbols[1]=GetSymbolByName(Symbol_02);
   //Symbols[2]=GetSymbolByName(Symbol_03);
//---
   for(int s=0; s<NUMBER_OF_SYMBOLS; s++)
     {
      grid[s].Init(s);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetRetracementRequiredTP()
  {
// Copy parameters specifying the retracements of the range between
//    achor and limit price per symbol over which
//       1+GridTakeProfit[*] many trades
//    are to be distributed.
//    It should be a range the price will likely retrace without
//    intermediate retracements hitting stop loss.
// NOTE: 1st risk-free conf is (TP,SL)=(4,1), if 50% retracements are
//       considered likely after break in we wish to hit 4 levels
//       until 50% which requires 9 trades in [21.4%,78.6%]
//       Apply a spread filter to the resulting max spacing.
//---
   RetracementRequiredTP[0]=InpRetracementRequiredTP_01;
   //RetracementRequiredTP[1]=InpRetracementRequiredTP_01;
   //RetracementRequiredTP[2]=InpRetracementRequiredTP_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetSpreadFilter()
  {
// After distributing
//    1+GridTakeProfit[*] many trades
// over the range between achor and limit price the resulting spacing
// is to be compared to the spread multiplied by this parameter
   SpreadFilter[0]=InpSpreadFilter_01;
   //SpreadFilter[1]=InpSpreadFilter_01;
   //SpreadFilter[2]=InpSpreadFilter_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetRetracmentForBreakOut()
  {
   RetracmentForBreakOut[0]=InpRetracmentForBrkOut_01;
   //RetracmentForBreakOut[1]=InpRetracmentForBrkOut_01;
   //RetracmentForBreakOut[2]=InpRetracmentForBrkOut_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetGridTakeProfit()
  {
   GridTakeProfit[0]=InpGridTakeProfit_01;
   //GridTakeProfit[1]=InpGridTakeProfit_01;
   //GridTakeProfit[2]=InpGridTakeProfit_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetGridStopLoss()
  {
   GridStopLoss[0]=InpGridStopLoss_01;
   //GridStopLoss[1]=InpGridStopLoss_01;
   //GridStopLoss[2]=InpGridStopLoss_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetGridParameters()
  {
   GetRetracementRequiredTP();
   GetSpreadFilter();
   GetRetracmentForBreakOut();
   GetGridTakeProfit();
   GetGridStopLoss();
//---
   for(int s=0; s<NUMBER_OF_SYMBOLS; s++)
     {
      //--- Variables depending on input parameters
      grid[s].SetupTrailingStops();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetDonchianChannelPeriods()
  {
   PeriodDC[0]=InpPeriod_01;
   //PeriodDC[1]=InpPeriod_01;
   //PeriodDC[2]=InpPeriod_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetDonchianChannelPercentsSell()
  {
   ChannelPercentSell[0]=InpChannelPercentSell_01;
   //ChannelPercentSell[1]=InpChannelPercentSell_01;
   //ChannelPercentSell[2]=InpChannelPercentSell_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetDonchianChannelPercentsBuy()
  {
   ChannelPercentBuy[0]=InpChannelPercentBuy_01;
   //ChannelPercentBuy[1]=InpChannelPercentBuy_01;
   //ChannelPercentBuy[2]=InpChannelPercentBuy_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetDonchianChannelParameters()
  {
   GetDonchianChannelPeriods();
   GetDonchianChannelPercentsSell();
   GetDonchianChannelPercentsBuy();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetStochasticMaxBuys()
  {
   StochMaxBuy[0]=InpStochMaxBuy_01;
   //StochMaxBuy[1]=InpStochMaxBuy_01;
   //StochMaxBuy[2]=InpStochMaxBuy_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetStochasticMinSells()
  {
   StochMaxBuy[0]=InpStochMaxBuy_01;
   //StochMaxBuy[1]=InpStochMaxBuy_01;
   //StochMaxBuy[2]=InpStochMaxBuy_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetStochasticKs()
  {
   StochK[0]=InpStochK_01;
   //StochK[1]=InpStochK_01;
   //StochK[2]=InpStochK_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetStochasticDs()
  {
   StochD[0]=InpStochD_01;
   //StochD[1]=InpStochD_01;
   //StochD[2]=InpStochD_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetStochasticSlowings()
  {
   StochSlowing[0]=InpStochSlowing_01;
   //StochSlowing[1]=InpStochSlowing_01;
   //StochSlowing[2]=InpStochSlowing_01;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void GetStochasticParameters()
  {
   GetStochasticMaxBuys();
   GetStochasticMinSells();
   GetStochasticKs();
   GetStochasticDs();
   GetStochasticSlowings();
  }
//+------------------------------------------------------------------+
//| Filling the indicator period array                               |
//+------------------------------------------------------------------+
void GetIndicatorPeriod()
  {
   //IndicatorPeriod[1]=IndicatorPeriod_02;
   //IndicatorPeriod[2]=IndicatorPeriod_03;
  }
//+------------------------------------------------------------------+
//| Filling the Take Profit array                                    |
//+------------------------------------------------------------------+
void GetTakeProfit()
  {
   //TakeProfit[1]=TakeProfit_02;
   //TakeProfit[2]=TakeProfit_03;
  }
//+------------------------------------------------------------------+
//| Filling the Stop Loss array                                      |
//+------------------------------------------------------------------+
void GetStopLoss()
  {
   //StopLoss[1]=StopLoss_02;
   //StopLoss[2]=StopLoss_03;
  }
//+------------------------------------------------------------------+
//| Filling the Trailing Stop array                                  |
//+------------------------------------------------------------------+
void GetTrailingStop()
  {
   //TrailingStop[1]=TrailingStop_02;
   //TrailingStop[2]=TrailingStop_03;
  }
//+------------------------------------------------------------------+
//| Filling the Reverse array                                        |
//+------------------------------------------------------------------+
void GetReverse()
  {
   //Reverse[1]=Reverse_02;
   //Reverse[2]=Reverse_03;
  }
//+------------------------------------------------------------------+
//| Filling the Lot array                                            |
//+------------------------------------------------------------------+
void GetLot()
  {
   //Lot[1]=Lot_02;
   //Lot[2]=Lot_03;
  }
//+------------------------------------------------------------------+
//| Filling the VolumeIncrease array                                 |
//+------------------------------------------------------------------+
void GetVolumeIncrease()
  {
   //VolumeIncrease[1]=VolumeIncrease_02;
   //VolumeIncrease[2]=VolumeIncrease_03;
  }
//+------------------------------------------------------------------+
//| Filling the VolumeIncreaseStep array                             |
//+------------------------------------------------------------------+
void GetVolumeIncreaseStep()
  {
   //VolumeIncreaseStep[1]=VolumeIncreaseStep_02;
   //VolumeIncreaseStep[2]=VolumeIncreaseStep_03;
  }
//+------------------------------------------------------------------+
//| Initializing the new bar array                                   |
//+------------------------------------------------------------------+
void InitializeArrayNewBar()
  {
//--- Initialize to zeros
   ArrayInitialize(new_bar,0);
//---
   for(int s=0; s<NUMBER_OF_SYMBOLS; s++)
     {
      //--- If trading for this symbol is allowed
      if(Symbols[s]!="")
         //--- Initialize the time of the current bar
         CheckNewBar(s);
     }
  }
//+------------------------------------------------------------------+
//| Initializing external parameter arrays                           |
//+------------------------------------------------------------------+
void InitializeInputParameters()
  {
   GetSymbols();
   GetGridParameters();              // SOS, TP, SL
   GetDonchianChannelParameters();   // Period, MinSell, MaxBuy
   GetStochasticParameters();        // K, D, Slow, MinSell, MaxBuy
   GetIndicatorPeriod();
   GetTakeProfit();
   GetStopLoss();
   GetTrailingStop();
   GetReverse();
   GetLot();
   GetVolumeIncrease();
   GetVolumeIncreaseStep();
  }
//+------------------------------------------------------------------+
//| Initializing arrays of indicator handles                         |
//+------------------------------------------------------------------+
void InitializeArrayHandles()
  {
   ArrayInitialize(spy_indicator_handles,INVALID_HANDLE);
   ArrayInitialize(signal_indicator_handles,INVALID_HANDLE);
  }
//+------------------------------------------------------------------+
//| Adding the specified symbol to the Market Watch window           |
//+------------------------------------------------------------------+
string GetSymbolByName(string symbol)
  {
   string symbol_name="";   // Symbol name on the server
//--- If an empty string is passed, return the empty string
   if(symbol=="")
      return("");
//--- Iterate over the list of all symbols on the server
   for(int s=0; s<SymbolsTotal(false); s++)
     {
      //--- Get the symbol name
      symbol_name=SymbolName(s,false);
      //--- If the required symbol is available on the server
      if(symbol==symbol_name)
        {
         //--- Select it in the Market Watch window
         SymbolSelect(symbol,true);
         //--- Return the symbol name
         return(symbol);
        }
     }
//--- If the required symbol cannot be found, return the empty string
   Print("The "+symbol+" symbol could not be found on the server!");
   return("");
  }
//+------------------------------------------------------------------+
