//--- Connection with the main file of the Expert Advisor
#include "../../Experts/FUGT/FUGT.mq5"
//--- Include custom libraries
#include "../../Include/FUGT/Enums.mqh"
#include "../../Include/FUGT/InitializeArrays.mqh"
#include "../../Include/FUGT/Errors.mqh"
#include "../../Include/FUGT/TradeSignals.mqh"
#include "../../Include/FUGT/TradeFunctions.mqh"
#include "../../Include/FUGT/ToString.mqh"
#include "../../Include/FUGT/Auxiliary.mqh"
//+------------------------------------------------------------------+
//| Getting agent handles by the specified symbols                   |
//+------------------------------------------------------------------+
void GetSpyHandles()
  {
//--- Iterate over all symbols
   for(int s=0; s<NUMBER_OF_SYMBOLS; s++)
     {
      //--- If trading for this symbol is allowed
      if(Symbols[s]!="")
        {
         //--- If the handle is yet to be obtained
         if(spy_indicator_handles[s]==INVALID_HANDLE)
           {
            //--- Get the indicator handle, the parameters are chart_id, custom_event_id, flag_event
            spy_indicator_handles[s]=iCustom(Symbols[s],_Period,"Shared Projects/FUGT/Indicators/FUGT/EventsSpy.ex5",ChartID(),0,CHARTEVENT_TICK);
            //--- If the indicator handle could not be obtained
            if(spy_indicator_handles[s]==INVALID_HANDLE)
               Print("Failed to install the agent on "+Symbols[s]+"");
           }
        }
     }
  }
//+------------------------------------------------------------------+
//| Getting indicator handles                                        |
//+------------------------------------------------------------------+
void GetIndicatorHandles()
  {
//--- Iterate over all symbols
   for(int s=0; s<NUMBER_OF_SYMBOLS; s++)
     {
      //--- If trading for this symbol is allowed
      if(Symbols[s]!="")
        {
         //--- If the handle is yet to be obtained
         if(signal_indicator_handles[s]==INVALID_HANDLE)
           {
            //--- Get the indicator handle
            signal_indicator_handles[s]=iCustom(Symbols[s],_Period,"Shared Projects/FUGT/Indicators/FUGT/donchian_two_levels.ex5",
                                                ChannelPercentBuy[s],      // InpChannelPercentBuy
                                                ChannelPercentSell[s],     // InpChannelPercentBuy
                                                PeriodDC[s],               // FiboPeriod
                                                HIGH_LOW,                  // Type of FiboPeriod points
                                                -2,                        // Margins
                                                0                          // Horizontal shift of the indicator in bars
                                               );
            //--- If the indicator handle could not be obtained
            if(signal_indicator_handles[s]==INVALID_HANDLE)
               Print("Failed to get the indicator handle for the symbol "+Symbols[s]+"!");
           }
        }
     }
  }
//+------------------------------------------------------------------+
//| Getting bar values                                               |
//+------------------------------------------------------------------+
void GetBarsData(int symbol_number)
  {
//--- Number of bars for copying to price data arrays
   int NumberOfBars=3;
//--- Reverse the indexing order (... 3 2 1 0)
   ArraySetAsSeries(open[symbol_number].value,true);
   ArraySetAsSeries(high[symbol_number].value,true);
   ArraySetAsSeries(low[symbol_number].value,true);
   ArraySetAsSeries(close[symbol_number].value,true);
//--- If the number of the obtained values is less than requested,
//    print the relevant message
//--- Get the closing price of the bar
   if(CopyClose(Symbols[symbol_number],_Period,0,NumberOfBars,close[symbol_number].value)<NumberOfBars)
     {
      Print("Failed to copy the values ("
            +Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the Close price array! "
            "Error ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
     }
//--- Get the opening price of the bar
   if(CopyOpen(Symbols[symbol_number],_Period,0,NumberOfBars,open[symbol_number].value)<NumberOfBars)
     {
      Print("Failed to copy the values ("
            +Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the Open price array! "
            "Error ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
     }
//--- Get the bar's high
   if(CopyHigh(Symbols[symbol_number],_Period,0,NumberOfBars,high[symbol_number].value)<NumberOfBars)
     {
      Print("Failed to copy the values ("
            +Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the High price array! "
            "Error ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
     }
//--- Get the bar's low
   if(CopyLow(Symbols[symbol_number],_Period,0,NumberOfBars,low[symbol_number].value)<NumberOfBars)
     {
      Print("Failed to copy the values ("
            +Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the Low price array! "
            "Error ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
     }
  }
//+------------------------------------------------------------------+
//| Getting indicator values                                         |
//+------------------------------------------------------------------+
bool GetIndicatorsData(int symbol_number)
  {
//--- Number of indicator buffer values for determining the trading signal
   int NumberOfValues=3;
//--- If the indicator handle has been obtained
   if(signal_indicator_handles[symbol_number]!=INVALID_HANDLE)
     {
      //--- Reverse the indexing order (... 3 2 1 0)
      ArraySetAsSeries(indicator_HH[symbol_number].value,true);
      //--- Get indicator values
      if(CopyBuffer(signal_indicator_handles[symbol_number],0,0,NumberOfValues,indicator_HH[symbol_number].value)<NumberOfValues)
        {
         Print("Failed to copy the values ("+
               Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the indicator array! Error ("+
               IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
         return(false);
        }
      //--- Reverse the indexing order (... 3 2 1 0)
      ArraySetAsSeries(indicator_UB[symbol_number].value,true);
      if(CopyBuffer(signal_indicator_handles[symbol_number],1,0,NumberOfValues,indicator_UB[symbol_number].value)<NumberOfValues)
        {
         Print("Failed to copy the values ("+
               Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the indicator array! Error ("+
               IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
         return(false);
        }
      //--- Reverse the indexing order (... 3 2 1 0)
      ArraySetAsSeries(indicator_LB[symbol_number].value,true);
      if(CopyBuffer(signal_indicator_handles[symbol_number],2,0,NumberOfValues,indicator_LB[symbol_number].value)<NumberOfValues)
        {
         Print("Failed to copy the values ("+
               Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the indicator array! Error ("+
               IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
         return(false);
        }
      //--- Reverse the indexing order (... 3 2 1 0)
      ArraySetAsSeries(indicator_LL[symbol_number].value,true);
      if(CopyBuffer(signal_indicator_handles[symbol_number],3,0,NumberOfValues,indicator_LL[symbol_number].value)<NumberOfValues)
        {
         Print("Failed to copy the values ("+
               Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the indicator array! Error ("+
               IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
         return(false);
        }
      //--- Reverse the indexing order (... 3 2 1 0)
      ArraySetAsSeries(indicator_BD[symbol_number].value,true);
      if(CopyBuffer(signal_indicator_handles[symbol_number],3,0,NumberOfValues,indicator_BD[symbol_number].value)<NumberOfValues)
        {
         Print("Failed to copy the values ("+
               Symbols[symbol_number]+"; "+TimeframeToString(_Period)+") to the indicator array! Error ("+
               IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
         return(false);
        }
      return(true);
     }
//--- If the indicator handle has not been obtained, then...
   else
     {
      // ...try to get it again
      GetIndicatorHandles();
     }
   return(false);
  }
//+------------------------------------------------------------------+
//| Determining trading signals                                      |
//+------------------------------------------------------------------+
bool CheckOpenLong(int symbol_number,double &price,double &sl,double &tp)
  {
   if(pos.exists)
     {
      //--- Never trade against the direction of a sell grid that already is being traded
      if(pos.type==POSITION_TYPE_SELL)
         return(false);
      //--- Only trade with the direction of a buy grid if there is enough profit from the last deal
      if(grid[symbol_number].CheckPyramidLong())
         return(grid[symbol_number].PyramidLongParams(price,sl,tp));
      //--- Don't add to long position without sufficient P/L
      return(false);
     }
//--- No open position means the price is ranging in the [0.0%,21.4%] or the [78.6%,100%] region.

   if(grid[symbol_number].CheckOuterLong())
      return(grid[symbol_number].OuterLongParams(price,sl,tp));

   if(Symbols[symbol_number]=="Ger30")
     {
      Sleep(1);
     }

   if(grid[symbol_number].CheckInnerLong())
      return(grid[symbol_number].InnerLongParams(price,sl,tp));
//---
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CheckOpenShort(int symbol_number,double &price,double &sl,double &tp)
  {
   if(pos.exists)
     {
      //--- Never trade against the direction of a buy grid that already is being traded
      if(pos.type==POSITION_TYPE_BUY)
         return(false);
      //--- Only trade with the direction of a sell grid if there is enough profit from the last deal
      if(grid[symbol_number].CheckPyramidShort())
         return(grid[symbol_number].PyramidShortParams(price,sl,tp));
      //--- Don't add to short position without sufficient P/L
      return(false);
     }
//--- No open position means the price is ranging in the [0.0%,21.4%] or the [78.6%,100%] region.

   if(grid[symbol_number].CheckOuterShort())
      return(grid[symbol_number].OuterShortParams(price,sl,tp));

   if(grid[symbol_number].CheckInnerShort())
      return(grid[symbol_number].InnerShortParams(price,sl,tp));
//---
   return(false);
  }
//+------------------------------------------------------------------+
